@@ -65,6 +65,11 @@ enum
 typedef struct _GMozillaEngine GMozillaEngine;
 typedef struct _GMozillaEngineClass GMozillaEngineClass;
 
+/** Context data structs */
+typedef struct _GMozillaEngineCtxUserData GMozillaEngineCtxUserData;
+typedef struct _GMozillaEngineCtxDomData GMozillaEngineCtxDomData;
+typedef struct _GMozillaEngineCtxMouseData GMozillaEngineCtxMouseData;
+
 /** @struct GMozillaEngine.
   * Defines a Mozilla Engine.
   */
@@ -75,12 +80,11 @@ struct _GMozillaEngine
     GObject* global;
 
     /* Context Info */
-    gint doc_index, x, y;
-    guint ctx_type;
-    guint moz_ctx_type;
-    gpointer ctx_node;
-    gpointer prev_ctx_node;
-    const gchar *ctx_url, *ctx_objurl, *ctx_docurl;
+    GMozillaEngineCtxUserData* user_ctx;
+    GMozillaEngineCtxDomData* dom_ctx;
+    GMozillaEngineCtxMouseData* mouse_ctx;
+
+    gint doc_index;
 
     GObject *ctx;
     GtkIMContext *gtkIMContext;
@@ -92,7 +96,7 @@ struct _GMozillaEngine
     gboolean im_do_manual_commit;
     gboolean im_key_pressed;
     guint32 mChromeMask;
-    gboolean isFkbShowed;
+    gboolean is_vkb_showed;
     gpointer last_event;
 
     /* small workaround for some JS progress values sent after finish loading */
@@ -119,7 +123,6 @@ struct _GMozillaEngine
     guint scroll_to_node_timer_id;
     guint vkb_up_timeout;
     gboolean mouse_down;
-    gboolean mouse_scroll;
     gboolean vkb_shown;
     GTimeVal key_press_time;
     GTimeVal key_release_time;
@@ -134,7 +137,6 @@ struct _GMozillaEngine
     GdkRegion *mBlockedUpdates;
     int mBlockedUpdatesSource;
     float mZoomValue;
-    GtkWidget *plug;
     gpointer select_node;
     GdkRectangle mWindowSize;
     GdkEventKey *prev_eventkey;
@@ -155,8 +157,21 @@ struct _GMozillaEngine
     GdkPoint mPrevScrollPos;
     gboolean emit_open_uri_after_click;
     gboolean user_iteraction_click;
-    gboolean skip_update_required;
     gboolean full_update_required;
+    gulong connectivity_signal_id;
+    gboolean mouse_up_received;
+
+    gpointer find;
+    gchar *prev_find_str;
+    gboolean recalculate_element;
+    gboolean menu_shown;
+
+    gint nsteps;
+    guint navigation_event_id;
+    gboolean is_back;
+
+    gboolean dom_focus_while_loading;
+    gboolean dom_focused_input;
 };
 
 /** @struct GMozillaEngineClass.
@@ -212,6 +227,7 @@ struct _GMozillaEngineClass
     void      (*notification_dialog)        (GMozillaEngine *, guint32 xid,  const gchar *);
     void      (*history_changed)            (GMozillaEngine *, guint, gint, gint);
     void      (*meta_refresh)               (GMozillaEngine *, const gchar *, gboolean, gint);
+    gboolean  (*add_search_provider)        (GMozillaEngine *, const gchar *);
 };
 
 typedef struct _SelectedItem
@@ -221,6 +237,40 @@ typedef struct _SelectedItem
   gboolean selected;
 } SelectedItem;
 
+/** Common context data */
+typedef struct _GMozillaEngineCtxData GMozillaEngineCtxData;
+struct _GMozillaEngineCtxData
+{
+    guint ctx_type;
+    guint moz_ctx_type;
+};
+
+/** User context data */
+struct _GMozillaEngineCtxUserData
+{
+    GMozillaEngineCtxData common_data;
+    // Referred URL strings are owned by this structure
+    gchar *ctx_url;
+    gchar *ctx_objurl;
+    gchar *ctx_docurl;
+    gint x;
+    gint y;
+};
+
+/** DOM context data */
+struct _GMozillaEngineCtxDomData
+{
+    GMozillaEngineCtxData common_data;
+    gboolean mouse_scroll;
+    gboolean mouse_up_received;
+};
+
+/** Mouse context data */
+struct _GMozillaEngineCtxMouseData
+{
+    GMozillaEngineCtxData common_data;
+};
+
 GType g_mozilla_engine_get_type (void);
 
 /** Creates a new mozilla engine instance
@@ -261,9 +311,6 @@ G_END_DECLS
 gint dom_mouse_long_press_cb (GtkMozEmbed *embed, void *event,
                                      GMozillaEngine *self);
 
-gint dom_mouse_down_cb (GtkMozEmbed *embed, void *event,
-                                     GMozillaEngine *self);
-
 guint g_mozilla_engine_get_zoom_level (GMozillaEngine *self);
 
 gboolean content_set_cursor(GMozillaEngine *self, GdkCursorType type);
@@ -280,7 +327,8 @@ void g_mozilla_add_notification_button(GMozillaEngine* self,
 
 void g_mozilla_show_notification(GMozillaEngine* self,
                                  gpointer notification_widget,
-                                 gpointer notification);
+                                 gpointer notification,
+                                 const char *title);
 
 void g_mozilla_remove_notification(GMozillaEngine* self,
                                    gpointer notification_widget);
@@ -305,6 +353,16 @@ GWebEngineEncoding g_mozilla_engine_get_encoding (GtkMozEmbed *);
 
 GtkIMContext* register_im_context(GMozillaEngine *self);
 
+gboolean ctx_text_input_or_password(GMozillaEngine *self);
+
+gboolean user_ctx_text_input_or_password(GMozillaEngine *self);
+
+gboolean ctx_text_input(GMozillaEngine *self);
+
+gboolean ctx_password(GMozillaEngine *self);
+
+gboolean ctx_multi(GMozillaEngine *self);
+
 void g_mozilla_engine_go_back(GMozillaEngine *self,
                               guint nsteps);
 void g_mozilla_engine_go_forward(GMozillaEngine *self,
