@@ -59,6 +59,8 @@
 #include "nsIPresShell.h"
 #include "gmozillaweb.h"
 
+#include <browser-translations.h>
+
 // Helper method for getting GMozillaEngine pointer from nsIDOMWindow
 GMozillaEngine* GetGMozEngineForDOMWindow(nsIDOMWindow* aDOMWindow, GObject** aMozEmbed)
 {
@@ -192,3 +194,15 @@ GetSendEventTools(GMozillaEngine *self, nsIWidget * *aWidget, nsIViewManager **a
     NS_IF_ADDREF(*aDOMWindow = window);
   return NS_OK;
 }
+
+PRBool
+CheckLogicalId(const nsAString& aSource, nsString& aTarget)
+{
+  if (StringBeginsWith(aSource, NS_LITERAL_STRING("weba_"))) {
+    nsCString string;
+    string.AssignLiteral(_(NS_ConvertUTF16toUTF8(aSource).get()));
+    CopyUTF8toUTF16(string, aTarget);
+    return PR_TRUE;
+  }
+  return PR_FALSE;
+}
