@@ -68,7 +68,6 @@ enum
     ON_SUBMIT_SIGNAL,
     ICON_SIGNAL,
     MODAL_DIALOG,
-    CONNECTIVITY_STATUS,
     ON_SERVICE_NOTIFY,
     LAST_SIGNAL                   /** < Last signal used */
 };
@@ -104,9 +103,10 @@ static void g_mozilla_web_class_init (GMozillaWebClass *);
 static void g_mozilla_web_set_default_config (GMozillaWeb *self);
 static gboolean g_mozilla_create_extensions_folder(void);
 void g_mozilla_connectivity_set_global (GMozillaWeb *self);
-void g_mozilla_connectivity_handle(GMozillaEngine *self, gint event);
 void g_mozilla_connectivity_set_proxy ();
 void g_mozilla_connectivity_unset_global(GMozillaWeb *self);
+static GWebStatus g_mozilla_web_store_transfers (GMozillaWeb *self);
+
 
 // GWebBus message handler
 static void web_bus_message_handler(guint msg_id, gpointer* msg, guint msg_len, GMozillaWeb* web);
@@ -142,14 +142,18 @@ g_mozilla_web_destroy (GMozillaWeb *self)
     TRACE_LOG();
     // Checking self pointer
     g_return_if_fail(self);
+    GList *temp_list = NULL, *list_next = NULL;
 
     if (self->window_list)
     {
-        GList *c = NULL;
+        temp_list = self->window_list;
         ULOG("Client init web without closing old %i engines", g_list_length(self->window_list));
-        for (c = (GSList*) self->window_list; c; c = c->next)
+        while (temp_list)
         {
-            g_web_engine_destroy(c->data);
+            list_next = temp_list->next;
+            if (temp_list->data)
+                g_web_engine_destroy(temp_list->data);
+            temp_list = list_next;
         }
         g_list_free(self->window_list);
     }
@@ -165,6 +169,8 @@ g_mozilla_web_destroy (GMozillaWeb *self)
         self->default_cursor = NULL;
     }
 
+    g_mozilla_web_store_transfers(self);
+
     g_mozilla_connectivity_unset_global(self);
 
     microb_eal_components_del();
@@ -185,13 +191,6 @@ g_mozilla_web_destroy (GMozillaWeb *self)
     }
 }
 
-static void
-connectivity_status_cb(GMozillaWeb *global, gint code, GMozillaEngine *self)
-{
-    if (self)
-        g_mozilla_connectivity_handle(self, code);
-}
-
 static GWebEngine*
 g_mozilla_web_new_web_engine_window_with_context (GMozillaWeb *self, gpointer context, gboolean leave_tracks, gboolean without_history)
 {
@@ -213,10 +212,8 @@ g_mozilla_web_new_web_engine_window_with_context (GMozillaWeb *self, gpointer co
     g_return_val_if_fail(engine, NULL);
 
     engine->global = G_OBJECT(self);
-    
-    self->window_list = g_list_append(self->window_list, engine);
 
-    g_signal_connect(G_OBJECT(self), G_WEBWIDGET_SIGNAL_CONNECTIVITY_STATUS, G_CALLBACK(connectivity_status_cb), engine);
+    self->window_list = g_list_append(self->window_list, engine);
 
     if (!self->is_configured) {
         g_mozilla_web_set_default_config (self);
@@ -845,11 +842,16 @@ g_mozilla_web_set_language (GMozillaWeb *self, const gchar* language)
     gchar *result_lang = moz_lang_short?moz_lang_short:G_MOZILLA_LANG_SHORT_DEFAULT;
     // g_strdup_printf("%s,%s,en-us,en", moz_lang_long?moz_lang_long:"", moz_lang_short?moz_lang_short:"");
     if (!g_mozilla_engine_set_pref(G_TYPE_STRING, G_MOZILLA_PREF_LANGUAGE, result_lang))
+    {
+        EAL_IF_GFREE(moz_lang_short);
+        EAL_IF_GFREE(moz_lang_long);
+        EAL_IF_GFREE(cur_lang);
         return G_WEB_ERR;
+    }
     g_mozilla_engine_set_pref(G_TYPE_STRING, G_MOZILLA_PREF_EAL_LOCALE, moz_lang_long?moz_lang_long:G_MOZILLA_EAL_LOCALE_DEFAULT);
     g_mozilla_engine_save_prefs();
-    EAL_GFREE(moz_lang_short);
-    EAL_GFREE(moz_lang_long);
+    EAL_IF_GFREE(moz_lang_short);
+    EAL_IF_GFREE(moz_lang_long);
     EAL_IF_GFREE(cur_lang);
 
     return G_WEB_STATUS_OK;
@@ -921,7 +923,7 @@ g_mozilla_web_free_memory (GMozillaWeb *self, gboolean progressive)
 }
 
 
-static GWebStatus
+GWebStatus
 g_mozilla_web_store_transfers (GMozillaWeb *self)
 {
     TRACE_LOG();
@@ -1792,15 +1794,6 @@ g_mozilla_web_class_init (GMozillaWebClass * klass)
                      6, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INT,
                      G_TYPE_INT, G_TYPE_INT, G_TYPE_INT);
 
-    g_mozilla_web_signals[CONNECTIVITY_STATUS] =
-        g_signal_new(G_WEBWIDGET_SIGNAL_CONNECTIVITY_STATUS,
-                     G_OBJECT_CLASS_TYPE (gobject_class),
-                     G_SIGNAL_RUN_LAST,
-                     G_STRUCT_OFFSET(GMozillaWebClass, connectivity_status),
-                     NULL, NULL,
-                     g_cclosure_marshal_VOID__INT,
-                     G_TYPE_NONE, 1, G_TYPE_INT);
-
     g_mozilla_web_signals[ON_SERVICE_NOTIFY] =
         g_signal_new(G_WEBWIDGET_SIGNAL_ON_SERVICE_NOTIFY,
                      G_OBJECT_CLASS_TYPE (gobject_class),
@@ -1824,7 +1817,7 @@ g_mozilla_web_ask_cookie (GMozillaEngine *engine,
     TRACE_LOG();
     g_return_if_fail (engine);
 
-    GWebCookieContext * ctx = (GWebCookieContext*) g_new0(GWebCookieContext*, 1);
+    GWebCookieContext * ctx = (GWebCookieContext*) g_new0(GWebCookieContext, 1);
     ctx->remember_decision = remember?*remember:FALSE;
     ctx->accept = action?*action:0;
 
@@ -1847,10 +1840,16 @@ g_mozilla_web_prompt (GMozillaEngine *engine, const gchar * dialog_title,
     TRACE_LOG ();
     g_return_val_if_fail (engine, FALSE);
 
+    // set flag to prevent any forbidden actions during popup
+    engine->dialog_is_active = TRUE;
+
     gchar *response = NULL;
     g_signal_emit_by_name (G_OBJECT (engine), G_WEBWIDGET_SIGNAL_PROMPT,
                            dialog_msg, *default_msg, &response, NULL);
 
+    // popup has been dismissed by now
+    engine->dialog_is_active = FALSE;
+
     if (*default_msg && response)
     {
         EAL_GFREE (*default_msg);
@@ -2228,11 +2227,19 @@ g_mozilla_web_set_default_config (GMozillaWeb *self)
     SET_DEFAULT_BOOL(G_MOZILLA_PREF_SUSPEND_PLUGINS, kTRUE)
     SET_DEFAULT_BOOL(G_MOZILLA_PREF_EXPAND_FRAMES, kTRUE)
     SET_DEFAULT_BOOL(G_MOZILLA_PREF_HIDE_FIXED_FRAMES, kTRUE)
-    g_mozilla_engine_get_pref(G_TYPE_BOOLEAN, G_MOZILLA_PREF_HIDE_FIXED_FRAMES, &self->mHideFixedFrames);
+
+    pref_bool = TRUE;
+    if (g_mozilla_engine_get_pref(G_TYPE_BOOLEAN, G_MOZILLA_PREF_HIDE_FIXED_FRAMES, &pref_bool))
+      self->mHideFixedFrames = pref_bool;
 
     // Permanently disable "Get Add-ons" from Options > Components menu
     g_mozilla_engine_set_pref(G_TYPE_BOOLEAN, G_MOZILLA_PREF_EXTENSIONS_GETADDONS_SHOWPANE, (gpointer)&kFALSE);
 
+    // Set force DPI value to avoid override by internal mozilla detector
+    // See Bug 449283 DPI on n810 is incorrect
+    int sysDPI = gdk_screen_get_resolution(gdk_screen_get_default());
+    SET_DEFAULT_INT_WEAK(G_MOZILLA_PREF_LAYOUT_CSS_DPI, sysDPI > 0 ? sysDPI : 96)
+
     pref_bool = TRUE;
     if (g_mozilla_engine_get_pref(G_TYPE_BOOLEAN, "gtkmozembed.pop_startup_on_last_window", &pref_bool) && pref_bool)
       self->shutdown_xpcom = FALSE;
@@ -2265,7 +2272,7 @@ g_web_new (void)
     g_signal_connect(G_OBJECT(gtk_moz_embed_single_get()), G_MOZILLA_NEW_WINDOW_ORPHAN,
                        GTK_SIGNAL_FUNC(new_window_orphan_cb), instance);
 
-    gWebGlobal = instance;
+    gWebGlobal = G_MOZILLA_WEB(instance);
     G_MOZILLA_WEB(instance)->mHideFixedFrames = TRUE;
 
     return instance;
@@ -2544,8 +2551,10 @@ web_bus_message_handler(guint msg_id, gpointer* msg, guint msg_len, GMozillaWeb*
             GMozillaEngine* engine = (GMozillaEngine*) m->engine_id;
             g_return_if_fail(validate_engine_pointer (web, engine));
             g_return_if_fail(G_IS_MOZILLA_ENGINE(engine));
-            g_mozilla_engine_send_key_event(engine, m->type, m->keyval, m->length, m->state,
-                                            m->hardware_keycode, m->group, m->is_modifier, m->time);
+            if (!g_mozilla_engine_send_key_event(engine, m->type, m->keyval, m->length, m->state,
+                                                 m->hardware_keycode, m->group, m->is_modifier, m->time))
+                // Engine did not consume the key, send it back to UI
+                g_web_bus_send_message(web->web_bus, web->ui_bus_name, msg_id, msg, msg_len);
             break;
         }
 
